/*

    ****************************************
    *   Computation of a Brownian motion   *
    ****************************************

    Copyright (C) 2003 Samuel Nicolay.
    email: <S.Nicolay@ulg.ac.be>


    This program is a free software, you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program (in a file named COPYRIGHT);
    if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
To compile, type something like 
gcc bm.c random.c utils.c -lm -o bm
*/

#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include<time.h>
#include"random.h" /*This generates a white noise*/
#include"utils.h"  /*Some useful easy functions*/

void chol_dec(float **output, long int size,float h,float var);
void help(char *file_name);
int main(int arg_c,char **arg_v){
  int j,no_name=0;
  long int size;
  float sum;
  char *name;
  FILE *output_file;

/*
************************************
*   First, we get the parameters   *
************************************
*/

  for (j=1;j<arg_c;j++) 
    if (arg_v[j][0]=='-')
      if (w1_is_w2("help",arg_v[j]+1,'-')){help(arg_v[0]);exit(0);}

  if (arg_c<2) error("Size not specified.\nTry the option '--help'.");
  else{
    if (string_is_int(arg_v[1])) size=(long int) string2float(arg_v[1]);
    else error("The first argument must be the size of the Bm.");
  }
  if (arg_c<3) {
    name = (char*) malloc(8*sizeof(char));
    name[0]='f';name[1]='b';name[2]='m';name[3]='.';name[4]='d';name[5]='a';name[6]='t';name[7]='\0';
    no_name=1;
    fprintf(stderr,"Output filename not specified.\nThe default name is \"%s\".\n",name);
  }
  else name=arg_v[2];



/*
*******************************************
*    We generate the fBm and we save it   *
*******************************************
*/

  if ((output_file=fopen(name,"w"))==NULL) fprintf(stderr,"Could not open \"%s\" for writing.\n",name);
  sum=0;
  for(j=0;j<size;j++){sum+=Grand(1.0);fprintf(output_file,"%f\n",sum);}
//  for(j=0;j<size;j++){fprintf(output_file,"%f\n",Grand(1.0));}
  fclose(output_file);

  if(no_name) free(name);
  return 0;
}

void help(char *file_name){
/*
*********************
*   Give the help   *
*********************
*/
  printf("%s size [output_filename]\n",file_name);
  printf("Generate Brownian motion of size \"size\".\n");
  printf("The output will be saved in the file \"output_filename\".\n");
}
