/*

    ****************************************************
    *   Computation of a fractionnal Brownian motion   *
    *      using the midpoint displacement method      *
    ****************************************************

    Copyright (C) 2003 Samuel Nicolay.
    email: <S.Nicolay@ulg.ac.be>


    This program is a free software, you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program (in a file named COPYRIGHT);
    if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
To compile, type something like 
gcc mpd.c random.c utils.c -lm -o fbm_mpd
*/

#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include<time.h>
#include"random.h" /*This generates a white noise*/
#include"utils.h"  /*Some useful easy functions*/

void build_fbm(float *output,long int size,float h);
void help(char *file_name);
int main(int arg_c,char **arg_v){
  int j,no_name=0;
  long int size;
  float h,*output,sum;
  char *name;
  FILE *output_file;

/*
************************************
*   First, we get the parameters   *
************************************
*/

  for (j=1;j<arg_c;j++) 
    if (arg_v[j][0]=='-')
      if (w1_is_w2("help",arg_v[j]+1,'-')){help(arg_v[0]);exit(0);}

  if (arg_c<2) error("Hurst exponent not specified.\nTry the option '--help'.");
  else {
    if (string_is_float(arg_v[1])) h=(float) string2float(arg_v[1]);
    else error("The first argument must be the Hurst exponent.");
    if(h>=1 || h<0) error("The Hurst exponent H must be such that 0<H<1.");
  }

  if (arg_c<3) error("Size not specified.");
  else{
    if (string_is_int(arg_v[2])) size=(long int) string2float(arg_v[2]);
    else error("The second argument must be the size.");
  }
  if (arg_c<4) {
    name = (char*) malloc(8*sizeof(char));
    name[0]='f';name[1]='b';name[2]='m';name[3]='.';name[4]='d';name[5]='a';name[6]='t';name[7]='\0';
    no_name=1;
    fprintf(stderr,"Output filename not specified.\nThe default name is \"%s\".\n",name);
  }
  else name=arg_v[3];



/*
********************************
*   Now, we generate the fBm   *
********************************
*/

  output=(float*) malloc(size*sizeof(float));
  build_fbm(output,size,h);

/*
**********************
*   And we save it   *
**********************
*/

  if ((output_file=fopen(name,"w"))==NULL) fprintf(stderr,"Could not open \"%s\" for writing.\n",name);
  sum=0;
  for(j=0;j<size;j++){fprintf(output_file,"%f\n",sqrt(size)*output[j]);}
  fclose(output_file);

/*
*******************************************
*   Time to end, so we free the vectors   *
*******************************************
*/

  if(no_name) free(name);
  free(output);

return 0;
}



void help(char *file_name){
/*
*********************
*   Give the help   *
*********************
*/
  printf("%s H size [output_filename]\n",file_name);
  printf("Generate exact fractional Brownian motion of size \"size\" with Hurst exponent \"H\" using the (rawest) midpoint displacement method.\n");
  printf("The output will be saved in the file \"output_filename\".\n");
  printf("This method is not very efficient.\n");
}

void build_fbm(float *output,long int size,float h){
  float new_point(float x1, float x2,long int n,float h);
  float *network;
  long int real_size,jump,t_pow,pos1,pos2,n,j;

  real_size=2;t_pow=1;
  while (real_size<size){
    real_size += t_pow;
    t_pow <<= 1;
  }
  t_pow >>= 1;
  real_size--;

  network=(float*) malloc(real_size*sizeof(float));
  network[0]=0;network[real_size]=Grand(1.0);
  jump=real_size;

  n=0;
  while(t_pow>0){
    pos1=0;pos2=jump;
    while(pos1<real_size){
       network[pos1+(jump>>1)]=new_point(network[pos1],network[pos2],n,h);
      pos1 += jump;
      pos2 += jump;
    }
    jump -= t_pow;
    t_pow >>= 1;
    n++;
  }

  for(j=0;j<size;j++)output[j]=network[j];
  free(network);
}

float new_point(float x1, float x2,long int n,float h){
 return 0.5*(x1+x2)+Grand(1.0)*sqrt((1-pow(2,2*h-2))/pow(2,2*n*h));
}
